#version 400 core
in vec3 frag_position;
in vec3 frag_normal;
in vec2 frag_uv;

out vec4 frag_color;

uniform vec3 view_position; 
uniform float brightness_setting;

// texture samplers
uniform sampler2D texture_diffuse1;
uniform sampler2D texture_specular1;
uniform sampler2D texture_normal1;
uniform sampler2D texture_height1;

void main()  
{
	//Hardcoded light source
	vec3 light_position = vec3(0, 10, 0); 
	vec3 light_color = vec3(1.0, 1.0, 1.0);
	vec3 texture_lightmap = vec3(1.0, 1.0, 1.0);

	// ambient part
	float ambient_factor = 1.0f * brightness_setting;
	vec3 ambient_part = ambient_factor * light_color;

	//diffuse part
	vec3 normal = normalize(frag_normal);
	vec3 light_direction = normalize(light_position -  frag_position);
	float diffuse_factor = max(dot(normal, light_direction), 0.0);
	vec3 diffuse_part = diffuse_factor * light_color;

	//specular part
	float specular_factor = 0.7;
	vec3 view_direction = normalize(view_position - frag_position);
	vec3 reflect_direction = reflect(-light_direction, normal);
	float specular = pow(max(dot(view_direction, reflect_direction), 0.0), 32);
	vec3 specular_part = specular_factor * specular * light_color;

	//texture part
	vec3 texture_color = texture(texture_diffuse1, frag_uv).rgb;
	// we are explointing the specular mtl slot to forward our lightmap
	texture_lightmap = texture(texture_specular1, frag_uv).rgb;

	// combine texture diffuse + lightmap
	diffuse_part = texture_color * texture_lightmap;
	
	//combining all parts
	vec3 result =  (ambient_part + specular_part) * diffuse_part;
	frag_color = vec4(result, 1.0);
}
